package org.ow2.petals.se.pojo.listener;

import java.lang.reflect.Method;
import java.util.logging.Level;

import javax.jbi.messaging.MessagingException;

import org.ow2.petals.component.framework.api.exception.FaultException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.listener.AbstractJBIListener;
import org.ow2.petals.se.pojo.Pojo;
import org.ow2.petals.se.pojo.PojoSE;

public class POJOJBIListener extends AbstractJBIListener {

    @Override
    protected void onJBIMessage(final Exchange exchange) {

        try {
            Pojo pojo = ((PojoSE) this.getComponent()).getPojos().get(exchange.getEndpointName());

            if (pojo != null) {
                try {
                    try {
                        setJbiListenerOnPojo(pojo);
                        pojo.callOnExchangeMethod(exchange);
                    } catch (final FaultException e) {
                        this.getLogger().log(Level.FINE, e.getMessage(), e);
                        exchange.setFault(e);
                    }
                } catch (final Exception e) {
                    this.getLogger().log(Level.SEVERE, e.getMessage(), e);
                    exchange.setError(e);
                }
            } else {
                this.getLogger().log(
                        Level.SEVERE,
                        "Failed to find the POJO instance matching the endpoint"
                                + exchange.getEndpointName() + "'");
                exchange.setError(new Exception(
                        "Failed to find the POJO instance matching the endpoint '"
                                + exchange.getEndpointName() + "'"));
            }
        } catch (MessagingException e) {
            exchange.setError(e);
        }
    }

    /**
     * Find a "setJBIListener(AbstractJBIListener jbiListener)" on the POJO and
     * set the AbstractJBIListener on it. The method is not mandatory.
     * 
     * @throws PEtALSComponentSDKException
     *             invocation failed
     */
    private void setJbiListenerOnPojo(Pojo pojo) throws Exception {

        // First, we set the SU classloader
        ClassLoader oldClassLoader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(pojo.getClasslLoader());

        try {
            Method setJbiListenerMethod = Pojo.findMethod(pojo.getClass(),
                    Pojo.JBI_LISTENER_SETTER, AbstractJBIListener.class);

            if (setJbiListenerMethod != null) {
                try {
                    setJbiListenerMethod.invoke(pojo, this);
                } catch (Throwable e) {
                    throw new Exception("Can not set the POJOJBIListener on the POJO", e);
                }
            }
        } finally {
            // We restore the thread classloader
            Thread.currentThread().setContextClassLoader(oldClassLoader);
        }
    }
}
