/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * ---------------------------------------------------------------------------
 * ---------------------------------------------------------------------------
 */
package org.ow2.petals.se.pojo.su;

import java.util.List;

import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.su.ServiceUnitListener;
import org.ow2.petals.component.framework.jbidescriptor.generated.Jbi;
import org.ow2.petals.component.framework.jbidescriptor.generated.Provides;
import org.ow2.petals.component.framework.su.ServiceUnitDataHandler;
import org.ow2.petals.component.framework.su.ServiceUnitManager;
import org.ow2.petals.component.framework.util.StringHelper;
import org.ow2.petals.se.pojo.Pojo;
import org.ow2.petals.se.pojo.PojoSE;

import static org.ow2.petals.component.framework.util.ClassLoaderUtil.createClassLoader;

/**
 * @author Frederic Gardes
 * 
 */
public class POJOSUListener implements ServiceUnitListener {

    private PojoSE pojoSE = null;

    public POJOSUListener(PojoSE pojoSE) {
        super();
        this.pojoSE = pojoSE;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.su.ServiceUnitListener#onSUDeployed
     * (java.lang.String, java.lang.String,
     * org.ow2.petals.component.framework.jbidescriptor.generated.Jbi)
     */
    public void onSUDeployed(String serviceUnitName, String suRootPath, Jbi descriptor)
            throws PEtALSCDKException {

        final List<Provides> providesList = descriptor.getServices().getProvides();
        if (providesList == null || providesList.size() != 1) {
            throw new PEtALSCDKException("One and only one provides have to be declared");
        }
        Provides provides = providesList.get(0);

        ServiceUnitDataHandler suDatahandler = ((ServiceUnitManager) pojoSE.getServiceUnitManager())
                .getServiceUnitDataHandlers().get(serviceUnitName);
        ConfigurationExtensions extensions = suDatahandler.getConfigurationExtensions(provides);
        
        Pojo pojo = null;
        if (extensions != null) {
            String className = extensions.get("class-name");
            if (className != null) {
                try {
                    pojo = createPojo(className, suDatahandler);
                    pojo.init();
                    this.pojoSE.getPojos().put(provides.getEndpointName(), pojo);
                } catch (Exception e) {
                    throw new PEtALSCDKException(e);
                }
            }

            if (pojo == null) {
                throw new PEtALSCDKException(
                        "'class-name' parameter not found in the JBI descriptor");
            }            
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.su.ServiceUnitListener#onSUStarted
     * (java.lang.String)
     */
    public void onSUStarted(String serviceUnitName) throws PEtALSCDKException {
        ServiceUnitDataHandler suDatahandler = ((ServiceUnitManager) pojoSE.getServiceUnitManager())
        .getServiceUnitDataHandlers().get(serviceUnitName);
        Provides provides = suDatahandler.getDescriptor().getServices().getProvides().get(0);
        
        Pojo pojo = pojoSE.getPojos().get(provides.getEndpointName());
        try {
            pojo.start();
        } catch (Exception e) {
            throw new PEtALSCDKException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.su.ServiceUnitListener#onSUStopped
     * (java.lang.String)
     */
    public void onSUStopped(String serviceUnitName) throws PEtALSCDKException {
        ServiceUnitDataHandler suDatahandler = ((ServiceUnitManager) pojoSE.getServiceUnitManager())
        .getServiceUnitDataHandlers().get(serviceUnitName);
        Provides provides = suDatahandler.getDescriptor().getServices().getProvides().get(0);
        
        
        Pojo pojo = pojoSE.getPojos().get(provides.getEndpointName());
        try {
            pojo.stop();
        } catch (Exception e) {
            throw new PEtALSCDKException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.su.ServiceUnitListener#onSUUndeployed
     * (java.lang.String, java.lang.String,
     * org.ow2.petals.component.framework.jbidescriptor.generated.Jbi)
     */
    public void onSUUndeployed(String serviceUnitName, String suRootPath, Jbi descriptor)
            throws PEtALSCDKException {
        ServiceUnitDataHandler suDatahandler = ((ServiceUnitManager) pojoSE.getServiceUnitManager())
        .getServiceUnitDataHandlers().get(serviceUnitName);
        Provides provides = suDatahandler.getDescriptor().getServices().getProvides().get(0);
        
        Pojo pojo = this.pojoSE.getPojos().remove(provides.getEndpointName());
        try {
            pojo.shutdown();
        } catch (Exception e) {
            throw new PEtALSCDKException(e);
        }
    }

    private Pojo createPojo(String className, ServiceUnitDataHandler suDatahandler)
            throws Exception {
        if (StringHelper.isNullOrEmpty(className)) {
            throw new Exception("Invalid class-name parameter");
        }

        // create a classloader from the SU path
        ClassLoader classLoader = createClassLoader(suDatahandler.getInstallRoot(), this.getClass()
                .getClassLoader());

        // load the pojo class from the classloader
        Class<?> clazz = null;
        try {
            clazz = classLoader.loadClass(className);
        } catch (ClassNotFoundException e) {
            throw new Exception(className + " class not found in the service-unit libraries", e);
        }

        // create the pojo
        Pojo pojo = new Pojo(clazz.newInstance(), classLoader, pojoSE.getContext(), pojoSE
                .getChannel(), pojoSE.getLogger());

        return pojo;
    }

}
